"""Public wing geometry API."""

from .internal.wing_canonical import (
    EvaluatedStation,
    SectionCurves,
    WingCanonical,
)
from .internal.wing_schema import (
    AirfoilRefSpec,
    BlendAirfoilRef,
    BlendMethod,
    CfdMeshSpec,
    ControlPointSpec,
    DegreesSpec,
    DistributionSpec,
    FitMethod,
    FitSpec,
    FrameSpec,
    JoinContinuity,
    JoinSideSpec,
    JoinSpec,
    MeshSpec,
    NurbsSpec,
    PanelSide,
    PanelSpec,
    PiecewiseLinearDistribution,
    ReferenceAxis,
    RefLineSpec,
    SingleAirfoilRef,
    SpacingMethod,
    SymmetryMode,
    UnitsSpec,
    VlmMeshSpec,
    WingDefinitionSpec,
    WingSpec,
)

__all__ = [
    "AirfoilRefSpec",
    "BlendAirfoilRef",
    "BlendMethod",
    "CfdMeshSpec",
    "ControlPointSpec",
    "DegreesSpec",
    "DistributionSpec",
    "EvaluatedStation",
    "FitMethod",
    "FitSpec",
    "FrameSpec",
    "JoinContinuity",
    "JoinSideSpec",
    "JoinSpec",
    "MeshSpec",
    "NurbsSpec",
    "PanelSide",
    "PanelSpec",
    "PiecewiseLinearDistribution",
    "RefLineSpec",
    "ReferenceAxis",
    "SectionCurves",
    "SingleAirfoilRef",
    "SpacingMethod",
    "SymmetryMode",
    "UnitsSpec",
    "VlmMeshSpec",
    "WingCanonical",
    "WingDefinitionSpec",
    "WingSpec",
]
